@@ -4,7 +4,7 @@
 ##                                                 ##
 #####################################################
 
-::Version::setSubversionId {$Id: plugins.tcl 10034 2008-05-28 14:36:41Z baaazen $}
+::Version::setSubversionId {$Id: plugins.tcl 11693 2009-10-12 00:31:42Z kakaroto $}
 
 proc plugins_log {plugin msg} {
 #return
@@ -326,7 +326,6 @@ namespace eval ::plugins {
 			foreach file [glob -nocomplain -directory $dir */plugininfo.xml] {
 				plugins_log core "Found plugin files in $file\n"
 				::plugins::LoadInfo $file
-				::plugins::LoadInfoAutoupdate $file
 			}
 		}
 
@@ -397,7 +396,7 @@ namespace eval ::plugins {
 		#<description_fr>the description in french</description_fr>
 		#The defaut description must be written in english
 		#The file should be encoded in utf-8
-		set langcode [::config::getGlobalKey language]
+		set langcode [string tolower [::config::getGlobalKey language]]
 		if { ($langcode != "en") && [info exists sdata(${cstack}:description_${langcode}) ] } {
 			set desc $sdata(${cstack}:description_${langcode})
 		} else {
@@ -410,17 +409,29 @@ namespace eval ::plugins {
 		set plugin_namespace $sdata(${cstack}:plugin_namespace)
 		set init $sdata(${cstack}:init_procedure)
 
+		if { ![info exists sdata(${cstack}:cvs_version)] } {
+			set cvs_version ""
+		} else {
+			set cvs_version $sdata(${cstack}:cvs_version)
+		}
+
 		if { ![info exists sdata(${cstack}:deinit_procedure)] } {
 			set deinit ""
 		} else {
 			set deinit $sdata(${cstack}:deinit_procedure)
 		}
-		
+
+		if {([getInfo $name plugin_version] != "" && [DetectNew $plugin_version [getInfo $name plugin_version]]) ||
+		    ([getInfo $name cvs_version] != "" && [DetectNew $cvs_version [getInfo $name cvs_version]])} {
+			# A previously detected plugin with the same name but a newer version is already loaded in memory.
+			return 0
+		}
 		set plugins(${name}_name) $name
 		set plugins(${name}_author) $author
 		set plugins(${name}_description) $desc
 		set plugins(${name}_amsn_version) $amsn_version
 		set plugins(${name}_plugin_version) $plugin_version
+		set plugins(${name}_cvs_version) $cvs_version
 		#dir is the path to pluginsinfo.xml, so we need to use [file dirname] to get the actual dir path
 		set plugins(${name}_plugin_file) [file join [file dirname $dir] $plugin_file]
 		set plugins(${name}_plugin_dir) [file dirname $dir]
@@ -506,9 +517,10 @@ namespace eval ::plugins {
 		bind $w.getmore <ButtonRelease> "launch_browser $::weburl/plugins.php?lang=$lang"
 
 
-		button $w.load -text "[trans load]" -command "::plugins::GUI_Load" -state disabled
-		button $w.config -text "[trans configure]" -command "::plugins::GUI_Config" -state disabled
-		button $w.close -text [trans close] -command "::plugins::GUI_Close"
+		frame $w.bf
+		button $w.bf.load -text "[trans load]" -command "::plugins::GUI_Load" -state disabled
+		button $w.bf.config -text "[trans configure]" -command "::plugins::GUI_Config" -state disabled
+		button $w.bf.close -text [trans close] -command "::plugins::GUI_Close"
  
 		#loop through all the plugins and add them to the list
 		foreach {plugin} [lsort -dictionary [array names ::plugins::plugins *_name]] {
@@ -539,9 +551,10 @@ namespace eval ::plugins {
 		pack $mF.author -padx 5 -anchor w
 		pack $mF.desc_title -padx 5 -anchor w
 		pack $mF.desc -anchor nw -expand true -fill x -padx 5
+		pack $w.getmore -side bottom -anchor e -padx 5
+		pack $w.bf.close $w.bf.config $w.bf.load -side right -anchor e -padx 5 -pady 5
+		pack $w.bf -side bottom -anchor e -padx 5 -pady 5
 		pack $w.sw -anchor w -side top -expand true -fill both
-		pack $w.getmore -side top -anchor e -padx 5
-		pack $w.close $w.config $w.load -padx 5 -pady 5 -side right -anchor se
 
 		moveinscreen $w 30
 		return
@@ -590,11 +603,11 @@ namespace eval ::plugins {
 		
 		# update the buttons
 
-		$w.config configure -state normal
+		$w.bf.config configure -state normal
 
 		if {[lsearch "$loadedplugins" $selection] != -1 } {
 			# if the plugin is loaded, enable the Unload button and update the colors
-			$w.load configure -state normal -text [trans unload] -command "::plugins::GUI_Unload"
+			$w.bf.load configure -state normal -text [trans unload] -command "::plugins::GUI_Unload"
 			$w.plugin_list itemconfigure [$w.plugin_list curselection] \
 				-bg [::skin::getKey extralistboxselectedbg] -fg [::skin::getKey extralistboxselected] \
 				-selectforeground [::skin::getKey extralistboxselected]
@@ -602,18 +615,18 @@ namespace eval ::plugins {
 			# if the plugin has a configlist, then enable configuration.
 			# Otherwise disable it
 			if {[info exists ::[getInfo $selection plugin_namespace]::configlist] == 1} {
-				$w.config configure -state normal
+				$w.bf.config configure -state normal
 			} else {
-				$w.config configure -state disabled
+				$w.bf.config configure -state disabled
 			}
 		} else { # plugin is not loaded
 			# enable the load button and disable config button and update color
-			$w.load configure -state normal -text "[trans load]" -command "::plugins::GUI_Load"
+			$w.bf.load configure -state normal -text "[trans load]" -command "::plugins::GUI_Load"
 			$w.plugin_list itemconfigure [$w.plugin_list curselection] \
 				-bg [::skin::getKey extrastdbgcolor] -fg [::skin::getKey extrastdtxtcolor] \
 				-selectforeground [::skin::getKey extraselectedtxtcolor]
 
-			$w.config configure -state disabled
+			$w.bf.config configure -state disabled
 		}
 	}
 
@@ -1210,6 +1223,9 @@ namespace eval ::plugins {
 
 		#save the loaded plugins
 		foreach {plugin} $loadedplugins {
+			if { [array names ::plugins::plugins ${plugin}_name] eq "" } {
+				continue
+			}
 			set namespace [getInfo $plugin plugin_namespace]
 			puts $file_id "\t<plugin>"
 			puts $file_id "\t\t<name>${plugin}</name>"
@@ -1233,6 +1249,9 @@ namespace eval ::plugins {
 		
 		#save the other plugins
 		foreach {plugin} [array names ::plugins::config] {
+			if { [array names ::plugins::plugins ${plugin}_name] eq "" } {
+				continue
+			}
 			puts $file_id "\t<plugin>"
 			puts $file_id "\t\t<name>${plugin}</name>"
 			puts $file_id "\t\t<loaded>false</loaded>"
@@ -1266,7 +1285,7 @@ namespace eval ::plugins {
 	# none
 	#
 	proc load_config {} {
-		global HOME password protocol
+		global HOME password
 		variable loadedplugins
 		foreach {plugin} $loadedplugins {
 			::plugins::UnLoadPlugin $plugin
@@ -1294,7 +1313,7 @@ namespace eval ::plugins {
 			#logic: if this is the first time the user logged in, then 
 			#the configuration list is empty which means load_config
 			#makes loadedplugins empty
-			set loadedplugins [list "Nudge" "Winks" "Cam Shooter" "remind"]
+			set loadedplugins [list "Nudge" "Winks" "Cam Shooter" "remind" "Notes" "ColoredNicks"]
 
 			status_log "Plugins System: load_config: No plugins.xml]\n" red
 		}
