@@ -6,10 +6,13 @@
 #	SOCKS5 support (integration) is experimental!!!
 #
 
-::Version::setSubversionId {$Id: proxy.tcl 10035 2008-05-28 14:41:14Z baaazen $}
+::Version::setSubversionId {$Id: proxy.tcl 11886 2010-01-11 01:43:13Z kakaroto $}
 
 package provide Proxy 0.1
-package require -exact http 2.4.4
+package require http
+package require SASL::NTLM;
+package require uri;
+package require base64;
 
 # This should be converted to a proper package, to use with package require
 source socks.tcl	;# SOCKS5 proxy support
@@ -77,7 +80,6 @@ proc HTTPsecureSocket { args } {
 		}
 
 		set result [lindex $reply 0]
-		set code [lindex [split $result { }] 1]
 
 		# be sure there's a valid response code
 		# We use a regexp because of some (or maybe only one) proxy returning "HTTP/1.0  200 .." with two spaces, 
@@ -88,8 +90,16 @@ proc HTTPsecureSocket { args } {
 		}
 
 		# now add tls to the socket and return it
+		fconfigure $socket -blocking 1 -buffering none -translation binary
+
+		::tls::import $socket
+
+		# We need to foce the handshake while the socket is blocking for tls to actually work
+		::tls::handshake $socket
+		
 		fconfigure $socket -blocking 0 -buffering none -translation binary
-		return [::tls::import $socket]
+		return $socket
+
 	}
 
 	# if not proxifying, just create a tls socket directly
@@ -110,14 +120,23 @@ proc SOCKSsecureSocket { args } {
 		set proxy_authenticate [expr [::config::getKey proxyauthenticate] == 1 ? 1 : 0]
 		set proxy_user [::config::getKey proxyuser]
 		set proxy_pass [::config::getKey proxypass]
-		set res [::Socks5::Init $socket $thost $tport $proxy_authenticate $proxy_user $proxy_pass]
+		if {[catch {set res [::Socks5::Init $socket $thost $tport $proxy_authenticate $proxy_user $proxy_pass]} res] } {
+			catch {close $socket}
+		}
 		if { $res != "OK" } {
 			return -code error $res
 		}
 
 		# now add tls to the socket and return it
+		fconfigure $socket -blocking 1 -buffering none -translation binary
+
+		::tls::import $socket
+
+		# We need to foce the handshake while the socket is blocking for tls to actually work
+		::tls::handshake $socket
+		
 		fconfigure $socket -blocking 0 -buffering none -translation binary
-		return [::tls::import $socket]
+		return $socket
 
 	}
 
@@ -139,7 +158,9 @@ proc SOCKSSocket { args } {
 		set proxy_authenticate [expr [::config::getKey proxyauthenticate] == 1 ? 1 : 0]
 		set proxy_user [::config::getKey proxyuser]
 		set proxy_pass [::config::getKey proxypass]
-		set res [::Socks5::Init $socket $thost $tport $proxy_authenticate $proxy_user $proxy_pass]
+		if {[catch {set res [::Socks5::Init $socket $thost $tport $proxy_authenticate $proxy_user $proxy_pass]} res] } {
+			catch {close $socket}
+		}
 		if { $res != "OK" } {
 			return -code error $res
 		}
@@ -149,7 +170,108 @@ proc SOCKSSocket { args } {
 	}
 
 	# if not proxifying, just create a socket directly
-	return [socket $thost $tport]
+	return [socket -async $thost $tport]
+}
+
+####################################################
+# All NTLM stuff adapted from autoproxy
+####################################################
+
+proc NTLMsecureSocket { args } {
+
+  upvar host thost
+  upvar port tport
+
+  set s [NTLMauthenticate $thost $tport]
+  return [::tls::import $socket]
+
+}
+
+proc NTLMsocket { args } {
+
+  upvar host thost
+  upvar port tport
+
+  return [::autoproxy::NTLMauthenticate $thost $tport]
+
+}
+
+
+proc NTLMCallback { context command args } {
+
+    switch -exact -- $command {
+        username    { return [::config::getKey proxyuser] }
+        password { return [::config::getKey proxypass] }
+        realm    { return "" }
+        hostname { return [info host] }
+        default  { return -code error unxpected }
+
+}
+
+proc NTLMauthenticate { thost tport } {
+
+  variable options
+  set phost $options(proxy_host)
+  set pport $options(proxy_port)
+
+  set s [socket $phost $pport]
+  fconfigure $s -blocking 1 -buffering line -translation crlf
+
+  set ctx [SASL::new -mechanism NTLM -callback [namespace origin NTLMCallback]]
+  while {1} {
+    set more_steps [SASL::step $ctx $challenge]
+    set response [SASL::response $ctx]
+    #puts "----"
+    #puts "Sending NTLM message to proxy"
+    #puts "NTLM key: [base64::encode -wrapchar {} $response]"
+    puts $s "CONNECT $thost:$tport HTTP/1.1\nHost: $phost\nProxy-Connection: Keep-Alive\nConnection: Keep-Alive\nCache-Control: no-cache\nPragma: no-cache\nProxy-Authorization: NTLM [base64::encode -wrapchar {} $response]\n"
+    #puts "----"
+    if {!$more_steps} {break}
+    #puts "Handling proxy reply"
+
+    set response {}
+    while {[set line [gets $s]] ne "" } {
+      append response $line\r\n
+      # break if headers done
+      if {$line == ""} {
+        break
+      }
+    }
+    set length {}
+    regexp {Content-Length: ([0-9]*)} $response -> length
+    #puts "response length: $length"
+    # puts $response
+    if {$length eq {} } {
+      set body [read $s]
+    } else {
+      set body [read $s [expr $length-1] ]
+    }
+    # puts $body
+    regexp {Proxy-Authenticate: NTLM (.*)\r\n} $response -> challenge
+    #puts "Server challenge: $challenge"
+    set challenge [base64::decode $challenge]
+   }
+   #puts "Handshake completed"
+   #puts "----"
+
+   SASL::cleanup $ctx
+
+   puts $socket ""
+   set reply ""
+   while {[gets $socket r] > 0} {
+      lappend reply $r
+   }
+
+   set result [lindex $reply 0]
+   if {! [regexp {^HTTP/1\.[01] +2[0-9][0-9]} $result]} {
+      return -code error $result
+   }
+
+   fconfigure $s -translation binary -buffering none -blocking 0
+   return $s
+
+}
+
 }
 
 ::snit::type Proxy {
@@ -186,6 +308,12 @@ proc SOCKSSocket { args } {
 
 	option -name
 	variable http_idlist
+	variable sockErr
+
+	method checkSocketErrors { sock } {
+		fileevent $sock writable {}
+		set sockErr [fconfigure $sock -err]
+	}
 
 	#Called to write some data to the connection
 	method write { sb data } {
@@ -246,29 +374,36 @@ proc SOCKSSocket { args } {
 			return -1
 		}
 
-		$sb configure -sock $sock
-		if { [$sb cget -proxy_host] != ""} {
-			if { [::config::getKey proxytype] == "http"} {
-				set res [$self ConnectHTTP $sock $proxy_serv $proxy_port $proxy_authenticate $proxy_user $proxy_password]
-				if { $res != "OK" } {
-					$sb configure -error_msg $res
-					return -1
-				}
-			} else {
-				set res [::Socks5::Init $sock $proxy_serv $proxy_port $proxy_authenticate $proxy_user $proxy_password]
-				if { $res != "OK" } {
-					$sb configure -error_msg $res
-					return -1
-				}
-			} 
-		}
-		fconfigure $sock -buffering none -translation binary -blocking 0
-		fileevent $sock readable [list $sb receivedData]
-		set connected_command [$sb cget -connected]
-		lappend connected_command $sock
-		fileevent $sock writable $connected_command
-		return 0
-
+		#set sockErr ""
+		#fileevent $sock writable [list $self checkSocketErrors $sock]
+		#tkwait variable [myvar sockErr]
+		#if { $sockErr == "" } {
+			$sb configure -sock $sock
+                	if { [$sb cget -proxy_host] != ""} {
+	                        if { [::config::getKey proxytype] == "http"} {
+	                                set res [$self ConnectHTTP $sock $proxy_serv $proxy_port $proxy_authenticate $proxy_user $proxy_password]
+	                                if { $res != "OK" } {
+	                                        $sb configure -error_msg $res
+	                                        return -1
+	                                }
+	                        } else {
+	                                set res [::Socks5::Init $sock $proxy_serv $proxy_port $proxy_authenticate $proxy_user $proxy_password]
+	                                if { $res != "OK" } {
+	                                        $sb configure -error_msg $res
+	                                        return -1
+	                                }
+	                        }
+	                }
+	                fconfigure $sock -buffering none -translation binary -blocking 0
+	                fileevent $sock readable [list $sb receivedData]
+	                set connected_command [$sb cget -connected]
+	                lappend connected_command $sock
+	                fileevent $sock writable $connected_command
+			return 0
+		#} else {
+		#	$sb configure -error_msg $sockErr
+		#	return -1
+		#}
 	}
 
 	method ConnectHTTP {sck addr port auth user pass} {
@@ -331,13 +466,7 @@ proc SOCKSSocket { args } {
 	}
 
 	method authInit {} {
-		global tlsinstalled login_passport_url
-
-		#Check if we need to install the TLS module
-		if { $tlsinstalled == 0 && [checking_package_tls] == 0} {
-			::autoupdate::installTLS
-			return -1
-		}
+		global login_passport_url
 
 		set proxy_host [ns cget -proxy_host]
 		set proxy_port [ns cget -proxy_port]
@@ -351,11 +480,27 @@ proc SOCKSSocket { args } {
 		}
 
 		if { [::config::getKey proxytype] == "http"} {
-			status_log "registering http secure socket "
-			if { [catch {http::register https 443 HTTPsecureSocket} res]} {
-				MSN::logout
-				MSN::reconnect "Proxy returned error: $res"
-				return -1
+			if { [::config::getKey proxyauthmethod] == "ntlm" } {
+				status_log "registering ntlm socket"
+				::http::config -proxyhost ""
+				if { [catch {http::register http 80 NTLMsocket} res]} {
+					MSN::logout
+	                                MSN::reconnect "Proxy returned error: $res"
+        	                        return -1
+				}
+				status_log "registering ntlm secure socket "
+                	        if { [catch {http::register https 443 NTLMsecureSocket} res]} {
+                        	        MSN::logout
+                                	MSN::reconnect "Proxy returned error: $res"
+                               	 return -1
+	                        }
+			} else {
+	                        status_log "registering http secure socket "
+	                        if { [catch {http::register https 443 HTTPsecureSocket} res]} {
+	                                MSN::logout
+	                                MSN::reconnect "Proxy returned error: $res"
+	                                return -1
+	                        }
 			}
 		} else {
 			# http://wiki.tcl.tk/2627 :(
@@ -770,7 +915,7 @@ proc SOCKSSocket { args } {
 		if { [$sb cget -proxy_host] == ""} {
 			set proxy_host [$sb cget -force_gateway_server]
 			if {$proxy_host == "" } {
-				set proxy_host "gateway.messenger.hotmail.com"
+				set proxy_host [::config::getKey start_gateway_server]
 			}
 			
 			set proxy_port 80
@@ -831,14 +976,14 @@ proc SOCKSSocket { args } {
 		if { $options(-direct) } {
 			set tmp_data "POST /gateway/gateway.dll?Action=open&Server=$server&IP=$remote_server HTTP/1.1\r\n"
 		} else {
-			set tmp_data "POST http://gateway.messenger.hotmail.com/gateway/gateway.dll?Action=open&Server=$server&IP=$remote_server HTTP/1.1\r\n"
+			set tmp_data "POST http://[::config::getKey start_gateway_server]/gateway/gateway.dll?Action=open&Server=$server&IP=$remote_server HTTP/1.1\r\n"
 		}
 	
 		append tmp_data "Accept: */*\r\n"
 		append tmp_data "Content-Type: text/xml; charset=utf-8\r\n"
 		append tmp_data "Content-Length: 0\r\n"
 		append tmp_data "User-Agent: MSMSGS\r\n"
-		append tmp_data "Host: gateway.messenger.hotmail.com\r\n"
+		append tmp_data "Host: [::config::getKey start_gateway_server]\r\n"
 		append tmp_data "Proxy-Connection: Keep-Alive\r\n"
 		append tmp_data "Connection: Keep-Alive\r\n"
 		append tmp_data "Pragma: no-cache\r\n"
